#include "pch.h"
#include "ui_text.h"
#include "ui_world.h"
#include <xray/render/base/world.h>
#include <xray/render/base/ui_renderer.h>

ui_text::ui_text(ui_world& w)
:m_ui_world(w),
m_font(NULL),
m_color(0xffffffff),
m_text_length(0)
{
	this->subscribe_event	(ev_text_changed, ui_event_handler(this, &ui_text::on_text_changed));
}

ui_text::~ui_text()
{}

void ui_text::set_font(enum_font fnt)
{
	XRAY_UNREFERENCED_PARAMETER	(fnt);
	m_font = m_ui_world.get_font_manager().get_font();
}

void ui_text::set_color(u32 clr)
{
	m_color = clr;
}

void ui_text::set_text(pcstr text)
{
	if(m_text_ != text)
	{
		m_text_ = text;
		process_event			(ev_text_changed, 0, 0);
	}
}

bool ui_text::on_text_changed(window* w, int p1, int p2)
{
	XRAY_UNREFERENCED_PARAMETERS	(w, p1, p2);

	m_text_length	= (u16)strings::length(get_text());
	return false;//non-exclusive handler
}

void ui_text::set_align(enum_text_align al)
{
	m_align			= al;
}

void ui_text::set_text_mode(enum_text_mode tm)
{
	m_mode			= tm;
}

#define IsSpace(ch)       ((ch) == ' ' || (ch) == '\t' || (ch) == '\r' || (ch) == '\n' || (ch) == ',' || (ch) == '.' || (ch) == ':' || (ch) == '!')

void parse_word(pcstr str, ui_font const* font, float& length, pcstr& next_word)
{
	length = 0.0f;
	while(*str)
	{
		length	+= font->get_char_tc(*str).z;
		if(IsSpace(*str))
			break;

		++str;
	}
	next_word = str+1;
}
void ui_text::fit_height_to_text()
{
	if(m_mode==tm_multiline)
	{
		pcstr ch				= get_text();
		u32 symb_count			= get_text_length();
		float const height		= m_font->get_height();
		float curr_word_len		= 0.0f;
		pcstr next_word			= NULL;
		float2 new_size			(0.0f, height);

		parse_word				(ch, m_font, curr_word_len, next_word);
		for(u32 i=0; i<symb_count; ++i, ++ch)
		{
			float3 const& tc	= m_font->get_char_tc(*ch);
			if(ch==next_word)
			{
				parse_word		(ch, m_font, curr_word_len, next_word);
				if(new_size.x+curr_word_len > get_size().x)
				{
					new_size.x	= 0.0f;
					new_size.y	+= height;
				}
			}
			new_size.x			+= tc.z;
		}
		w()->set_size(float2(get_size().x, new_size.y));
	}
}
void ui_text::split_and_set_text(pcstr str, float const width, pcstr& ret_str)
{
	pcstr tmp_str			= str;
	float curr_word_len		= 0.0f;
	string1024				text;
	pcstr next_word			= NULL;
	float size				= 0.0f;

	parse_word				(str, m_font, curr_word_len, next_word);
	while(size+curr_word_len < width && strings::length(tmp_str))
	{
		if(tmp_str==next_word)
		{
			size			+= curr_word_len;
			parse_word		(tmp_str, m_font, curr_word_len, next_word);
		}
		else
			++tmp_str;
	}
	strings::copy_n			(text, sizeof(text), str, strings::length(str)-strings::length(tmp_str));
	set_text(text);
	strings::length(next_word)>0?ret_str = tmp_str:ret_str = NULL;
}

void ui_text::draw(xray::render::world& w)
{
	draw_internal(w, 0, 0, 0xffff0000);
}

float2 ui_text::measure_string() const
{
	float2			result;
	result.y		= get_size().y;
	result.x		= calc_string_length(*m_font, get_text());
	return			result;
}

void ui_text::draw_internal(xray::render::world& render_world, int const& idx1, int const& idx2, u32 const& sel_clr)
{
	ui_window::draw			(render_world);
	if(0==get_text_length())
		return;

	ASSERT					(m_font, "font not assigned");
	int prim_type			= 0;
	int point_type			= 0;

	float2  pos				= get_position();
	float2 pos_rt			(0.0f, 0.0f);
	client_to_screen		(*this, pos);		//need absolute position

	u32 symb_count			= get_text_length();
	u32 max_verts			= symb_count*4; // use TriangleList+ IndexBuffer(0-1-2, 3-2-1)

	pcstr ch				= get_text();
	float const height		= m_font->get_height();
	float height_ts			= m_font->get_height_ts();

	float curr_word_len		= 0.0f;
	pcstr next_word			= NULL;

	if(m_mode==tm_multiline)
		parse_word			(ch, m_font, curr_word_len, next_word);

	xray::render::ui::command* cmd = render_world.ui().create_command(max_verts, prim_type, point_type);
	for(u32 i=0; i<symb_count; ++i, ++ch)
	{
		u32 clr = (i>=(u32)idx1 && i<(u32)idx2)?sel_clr:m_color;

		float3 uv			= m_font->get_char_tc_ts(*ch);
		float3 const& tc	= m_font->get_char_tc(*ch);

		if((m_mode==tm_multiline) && (ch==next_word))
		{
			parse_word		(ch, m_font, curr_word_len, next_word);
			if(pos_rt.x+curr_word_len >get_size().x)
			{
				pos_rt.x		= 0.0f;
				pos_rt.y		+= height;
			}
		}

		cmd->push_point		(pos.x+pos_rt.x,		pos.y+pos_rt.y+height,	0.0f, clr, uv.x,		uv.y+height_ts);
		cmd->push_point		(pos.x+pos_rt.x,		pos.y+pos_rt.y,			0.0f, clr, uv.x,		uv.y);
		cmd->push_point		(pos.x+pos_rt.x+tc.z,	pos.y+pos_rt.y+height,	0.0f, clr, uv.x+uv.z,	uv.y+height_ts);
		cmd->push_point		(pos.x+pos_rt.x+tc.z,	pos.y+pos_rt.y,			0.0f, clr, uv.x+uv.z,	uv.y);

		pos_rt.x			+= tc.z;
	}
//	cmd->set_shader();
//	cmd->set_geom();
//	w.ui().destroy_command(cmd);
	render_world.push_command		(cmd);
}
